package travelplanner;

import de.iflow.components.IFlowApplication;
import de.iflow.util.IFlowUtil;

public class TravelPlanner extends IFlowApplication {

	private FlightOffer flightOffer;

	private static TravelPlanner instance;

	// constructor can't be private in android
	public TravelPlanner() {
		// Activity is automatically called/created by the Android Framework
		// set reference to object with keyword this
		instance = this;
	}

	// then it's not really a singleton pattern...
	public static TravelPlanner getInstance() {
		// in android instance won't be null, if activity is created
		if (instance == null) {
			instance = new TravelPlanner();
		}
		return instance;
	}

	/*
	 * Implement abstract methods from inherited abstract parent
	 */

	@Override
	public void init() {
		IFlowUtil.log("init");
		User.getInstance().getInputForRetInput();
	}

	@Override
	protected void storeData() {
		storeDataInSharedPreferences(TravelPlanner.class.getSimpleName(), "flightoffer", flightOffer);
	}

	@Override
	protected void restoreData() {
		flightOffer = (FlightOffer) restoreDataFromSharedPreferences(TravelPlanner.class.getSimpleName(), "flightoffer", FlightOffer.class);
		if (flightOffer != null) {
			IFlowUtil.log("ID" + flightOffer.getId());
		}
	}

	@Override
	protected void handleIntentExtras() {
		IFlowUtil.log("handleIntentExtras");
		if (getIFlowMessage("DeclassifiedCCD", DeclassifiedCCD.class) != null) {
			DeclassifiedCCD message = (DeclassifiedCCD) getIFlowMessage("DeclassifiedCCD", DeclassifiedCCD.class);
			declassifiedCCD(message);
		}
	}

	/*
	 * TravelPlanner
	 */
	public void retInput(RetInput message) {
		IFlowUtil.log("retInput");
		RequestData requestData = message.getRequestData();
		TravelAgency.getInstance().getFlightOffers(new GetFlightOffers(requestData));
	}

	public void retFlightOffers(RetFlightOffers message) {
		IFlowUtil.log("retFlightOffers");
		User.getInstance().getSingleSelectionForRetSingleSelection(new GetSingleSelection(message.getFlightOffers()));
	}

	public void retSingleSelection(RetSingleSelection message) {
		IFlowUtil.log("retSingleSelection");
		flightOffer = message.getFlightOffer();
		CreditCardCenter.getInstance().releaseCCD(new ReleaseCCD(flightOffer.getAirline()));
	}

	public void declassifiedCCD(DeclassifiedCCD message) {
		IFlowUtil.log("declassifiedCCD");
		CreditCardData ccd_decl = message.getCcd();
		Airline airline = flightOffer.getAirline();
		airline.bookFlightOffer(new BookFlightOffer(flightOffer.getId(), ccd_decl));
	}

	public void okConfirmBooking(OkConfirmBooking message) {
		IFlowUtil.log("okConfirmBooking");
		User.getInstance().ok(new Ok());
	}
}
